/**
 * Project: AVR ATtiny USB Tutorial at http://codeandlife.com/
 * Author: Joonas Pihlajamaa, joonas.pihlajamaa@iki.fi
 * Base on V-USB example code by Christian Starkjohann
 * Copyright: (c) 2008 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v3 (see License.txt)
 */
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "usbdrv.h"
#include <util/delay.h>
#include "rc5.c"
#include "keys.h"     /*keys table */


//static uchar    reportBuffer[4];
PROGMEM char usbHidReportDescriptor[159] = { /* USB report descriptor, size must match usbconfig.h */
/*mouse*/
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x02,                    // USAGE (Mouse)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x09, 0x01,                    //   USAGE (Pointer)
    0xA1, 0x00,                    //   COLLECTION (Physical)
	0x85, 0x01,                    //     REPORT_ID (1)
    0x05, 0x09,                    //     USAGE_PAGE (Button)
    0x19, 0x01,                    //     USAGE_MINIMUM 1
    0x29, 0x03,                    //     USAGE_MAXIMUM 3
    0x15, 0x00,                    //     LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //     LOGICAL_MAXIMUM (1)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x75, 0x01,                    //     REPORT_SIZE (1)
    0x81, 0x02,                    //     INPUT (Data,Var,Abs)
    0x95, 0x01,                    //     REPORT_COUNT (1)
    0x75, 0x05,                    //     REPORT_SIZE (5)
    0x81, 0x03,                    //     INPUT (Const,Var,Abs)
	0x05, 0x01,                    //     USAGE_PAGE (Generic Desktop)
    0x09, 0x30,                    //     USAGE (X)
    0x09, 0x31,                    //     USAGE (Y)
    0x09, 0x38,                    //     USAGE (Wheel)
    0x15, 0x81,                    //     LOGICAL_MINIMUM (-127)
    0x25, 0x7f,                    //     LOGICAL_MAXIMUM (127)
    0x75, 0x08,                    //     REPORT_SIZE (8)
    0x95, 0x03,                    //     REPORT_COUNT (3)
    0x81, 0x06,                    //     INPUT (Data,Var,Rel)
    0xC0,                          //   END_COLLECTION
    0xC0,                          // END COLLECTION
/*consumer keyboard */
    0x05, 0x0c,                    // USAGE_PAGE (Consumer Devices)
    0x09, 0x01,                    // USAGE (Consumer Control)
    0xa1, 0x01,                    // COLLECTION (Application)
	0x85, 0x02,                    // REPORT_ID (2)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x02,                    //   LOGICAL_MAXIMUM (3)
    0x75, 0x01,                    //   REPORT_SIZE (1)    - each field occupies 1 bit
	0x95, 0x05,                    //   REPORT_COUNT (5)
    0x09, 0xb5,                    //   USAGE (Scan Next Track)
    0x09, 0xb6,                    //   USAGE (Scan Previous Track)
    0x09, 0xb7,                    //   USAGE (Stop)
    0x09, 0xcd,                    //   USAGE (Play/Pause)
    0x09, 0xe2,                    //   USAGE (Mute)
    0x81, 0x06,                    //   INPUT (Data,Var,Rel)  - relative inputs
    0x95, 0x02,                    //   REPORT_COUNT (2)
    0x09, 0xe9,                    //   USAGE (Volume Up)
    0x09, 0xea,                    //   USAGE (Volume Down)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)  - absolute inputs
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x81, 0x01,                    //   INPUT (Cnst,Ary,Abs)
    0xc0,							// END_COLLECTION
/*standard keyboard*/
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x06,                    // USAGE (Keyboard)
    0xa1, 0x01,                    // COLLECTION (Application)
	0x85, 0x03,                    // REPORT_ID (3)
	0x05, 0x07,                    //   USAGE_PAGE (Keyboard)
    0x19, 0xe0,                    //   USAGE_MINIMUM (Keyboard LeftControl)
    0x29, 0xe7,                    //   USAGE_MAXIMUM (Keyboard Right GUI)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x08,                    //   REPORT_COUNT (8)
    0x81, 0x02,                    //   INPUT (Data,Var,Abs)    
    0x95, 0x01,                    //   REPORT_COUNT (1)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x25, 0x65,                    //   LOGICAL_MAXIMUM (101)
    0x19, 0x00,                    //   USAGE_MINIMUM (Reserved (no event indicated))
    0x29, 0x65,                    //   USAGE_MAXIMUM (Keyboard Application)
    0x81, 0x00,                    //   INPUT (Data,Ary,Abs)
    0xc0,                          // END_COLLECTION
/*system control*/
    0x05, 0x01,                    // USAGE_PAGE (Generic Desktop)
    0x09, 0x80,                    // USAGE (System Control)
    0xa1, 0x01,                    // COLLECTION (Application)
	0x85, 0x04,                    // REPORT_ID (4)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x25, 0x01,                    //   LOGICAL_MAXIMUM (1)
    0x19, 0x81,                    //   USAGE_MINIMUM (System Power Down)
    0x29, 0x83,                    //   USAGE_MAXIMUM (System Wake Up)
    0x75, 0x01,                    //   REPORT_SIZE (1)
    0x95, 0x03,                    //   REPORT_COUNT (3)
    0x81, 0x06,                    //   INPUT (Data,Var,Rel)
    0x95, 0x05,                    //   REPORT_COUNT (5)
    0x81, 0x01,                    //   INPUT (Cnst,Ary,Abs)
	0xc0                           // END_COLLECTION
};

typedef struct{
	uchar	ID;
    uchar   byte1;
    uchar   byte2;
    uchar   byte3;
    uchar   byte4;
} report;
static report reportBuffer;
//static uchar    idleRate;   /* repeat rate for keyboards, never used for mice */

usbMsgLen_t usbFunctionSetup(uchar data[8]) {
    usbRequest_t *rq = (void *)data;

    // The following requests are never used. But since they are required by
    // the specification, we implement them in this example.
    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS) {
        if(rq->bRequest == USBRQ_HID_GET_REPORT) {  
            // wValue: ReportType (highbyte), ReportID (lowbyte)
            usbMsgPtr = (void *)&reportBuffer; // we only have this one
            return sizeof(reportBuffer);
        } else if(rq->bRequest == USBRQ_HID_GET_IDLE) {
//            usbMsgPtr = &idleRate;
            return 1;
        } else if(rq->bRequest == USBRQ_HID_SET_IDLE) {
//            idleRate = rq->wValue.bytes[1];
        }
    }
    
    return 0; // by default don't return any data
}

int main() {

	uchar i, lastkey = 0, keys;

#define LED_PIN (1<<PD3)
DDRD |= LED_PIN;

#define LED_ON PORTD |= LED_PIN //waczenie LED
#define LED_OFF PORTD &= ~LED_PIN //wyaczenie LED

LED_ON;
_delay_ms(500);
LED_OFF;

command=0xff;
address=0xff;

ir_init();	/* inicjalizacja dekodowania IR */
	
    wdt_enable(WDTO_1S); // enable 1s watchdog timer

    usbInit();
	
    usbDeviceDisconnect(); // enforce re-enumeration
    for(i = 0; i<250; i++) { // wait 500 ms
        wdt_reset(); // keep the watchdog happy
        _delay_ms(2);
    }
    usbDeviceConnect();
	
    sei(); // Enable interrupts after re-enumeration


/**************************************************************************/	
    while(1) {
        wdt_reset(); 
        usbPoll();
        
        if(usbInterruptIsReady()) { 
           

//************************************************************
					if (lastkey == 0)
						{
				
						if (rc5_data) {	/* jeli odebrano prawidowe kody z pilota */
						keys = 0;
						lastkey = 2;

						
								if( !address ) 		/* jeli adres == 0 */
							//	if (address == 1)  		// jeli inny adres pilota - zamiast cyfry 1 wpisa waciw   
											{

													LED_ON; //waczenie LED
																								
											reportBuffer.ID    = keys_table [command] [0];
											reportBuffer.byte1 = keys_table [command] [1];
											reportBuffer.byte2 = keys_table [command] [2]; 
											reportBuffer.byte3 = keys_table [command] [3];
											reportBuffer.byte4 = keys_table [command] [4];
						
							/* wyzerowanie flagi odbioru oraz wartoci odebranych kodw */
							rc5_data=0;
							command=0xff;
							address=0xff;
										}
		 							}
						}

						usbPoll();

							if (lastkey)
							{
							lastkey = lastkey - 1;
					
				//*************************************************************
		   
				usbSetInterrupt((void *)&reportBuffer, sizeof(reportBuffer));
							//		reportBuffer.ID    = 0;
									reportBuffer.byte1 = 0;
									reportBuffer.byte2 = 0; 
            						reportBuffer.byte3 = 0;
            						reportBuffer.byte4 = 0;

				_delay_ms(50); // czekanie 50ms zanim zgasimy led

			
        		LED_OFF; //wyczenie LED
										}
       }
	
    
	}
    return 0;
}
